
// Persistence of Vision Ray Tracer Scene Description File
// File: TesTree1.pov
// Vers: 3
// Desc: Test file for Tree_V03.inc, Tree Generation include for POV 3.0
// Date: Jun 1997
// Auth: Gianluca Terenziani


// ==== Standard POV-Ray Includes ====
#include "colors.inc"	// Standard Color definitions
#include "textures.inc"	// Standard Texture definitions



// Set a color of the background (sky)
background { color red 0.1 green 0.3 blue 0.8 }


camera
{
  location  <0.0 , 2.0 ,-12.0>
  right 4/3*x
  up y
  look_at   <4.0 , 3.0 , 0.0>
}


// create a regular point light source
light_source
{
  0*x // light's position (translated below)
  color red 1.0  green 1.0  blue 1.0  // light's color
  translate <-20, 40, -20>
}


// An infinite planar surface
// plane {<A, B, C>, D } where: A*x + B*y + C*z = D
plane
{
  y, // <X Y Z> unit surface normal, vector points "away from surface"
  0.0 // distance from the origin in the direction of the surface normal
  texture { pigment {color rgb <.1,.5,.1>} finish {crand .3}}
}


#declare Leaf =
union {
cone {<0,0,0>,.015,<0,.5,0>,0 pigment {color rgb <1,.7,.5> } }

// arbitrary X,Y FINITE (no CSG) shape
// A complex example for a polygon is the letter "P":
polygon
{
  13, // number of points
  <0, .1>, <.1, .15>, <.2, .2>, <.19, .25>, <.15, .32>, <.08, .38>, <0, .5>, // list of <u,v> points
  <-.08, .38>, <-.15, .32>, <-.19, .25>,  <-.2, .2>, <-.1, .15>,<0, .1>// list of <u,v> points
   pigment {color rgb <.7,1,.7> }
}

}


// Default Tree (need only the include)
#include "Tree_V03.inc"

// Custom Tree
// Reinitialize Randomizer seed
#declare T_Seed1 = 78
#declare T_Seed2 = 33
#declare T_Seed3 = 12
#declare T_Seed4 = 780
// Declare it only for fun, value are equal to default
#declare T_Subdivision = 3
#declare T_Branch = 3
#declare T_Gravity = 20
#declare T_InitLenght = 3
#declare T_MinLenght = 1.5
// Subdivide the branch and increase the irregularity
#declare T_SBranch = 4
#declare T_Contort = 7
// Setting up the size of the Branch of the tree
#declare T_InitSize = .1
#declare T_MinSize = .02
// Specify Leaf Obj
#declare T_LeafObj = Leaf
// and Growing direction
#declare T_Direction = <0.1,1,0>
// Translate it
#declare T_Translate = <8,0,0>
// Declare new subbranching parameters
#declare T_SubRnd1 = .15
#declare T_SubRnd2 = 0
// And MAKE IT !
#include "Tree_V03.inc"

