
// Persistence of Vision Ray Tracer Scene Description File
// File: TesTree3.pov
// Vers: 3
// Desc: Test file for Tree_V03.inc, Tree Generation include for POV 3.0
// Date: Jun 1997
// Auth: Gianluca Terenziani


// ==== Standard POV-Ray Includes ====
#include "colors.inc"	// Standard Color definitions
#include "textures.inc"	// Standard Texture definitions



// Set a color of the background (sky)
background { color red 0.1 green 0.3 blue 0.8 }


camera
{
  location  <4.0 , 2.0 ,-15.0>
  right 4/3*x
  up y
  look_at   <4.0 , 3.0 , 0.0>
}


// create a regular point light source
light_source
{
  0*x // light's position (translated below)
  color red 1.0  green 1.0  blue 1.0  // light's color
  translate <-20, 40, -20>
}


// An infinite planar surface
// plane {<A, B, C>, D } where: A*x + B*y + C*z = D
plane
{
  y, // <X Y Z> unit surface normal, vector points "away from surface"
  0.0 // distance from the origin in the direction of the surface normal
  texture { pigment {color rgb <.1,.5,.1>} finish {crand .3}}
}


#declare Leaf =
union {
cone {<0,0,0>,.015,<0,.5,0>,0 pigment {color rgb <.7,.7,.5> } }

// arbitrary X,Y FINITE (no CSG) shape
// A complex example for a polygon is the letter "P":
polygon
{
  13, // number of points
  <0, .1>, <.1, .15>, <.2, .2>, <.19, .25>, <.15, .32>, <.08, .38>, <0, .5>, // list of <u,v> points
  <-.08, .38>, <-.15, .32>, <-.19, .25>,  <-.2, .2>, <-.1, .15>,<0, .1>// list of <u,v> points
   pigment {color rgb <.7,1,.7> }
}

}


// Tree 1
#declare T_Seed1 = 12
#declare T_Seed2 = 13
#declare T_Seed3 = 14
#declare T_Seed4 = 15
#declare T_Subdivision = 3
#declare T_Branch = 3
#declare T_SBranch = 6
#declare T_LeafObj = object { Leaf scale .8 }
#declare T_Gravity = 4
#declare T_Contort = 4
#declare T_InitLenght = 2.5
#declare T_MinLenght = 3
#declare T_InitSize = .4
#declare T_MinSize = .02
#declare T_SubRnd1 = .3
#declare T_SubRnd2 = .3
#include "Tree_V03.inc"




// Tree 2
#declare T_Seed1 = 38
#declare T_Seed2 = 43
#declare T_Seed3 = 62
#declare T_Seed4 = 80
#declare T_Subdivision = 3
#declare T_Branch = 4
#declare T_SBranch = 4
#declare T_Gravity = 40
#declare T_Contort = 5
#declare T_InitLenght = 3
#declare T_MinLenght = 1.5
#declare T_InitSize = .1
#declare T_MinSize = .05
#declare T_LeafObj = Leaf
#declare T_Direction = <0.1,1,0>
#declare T_Translate = <8,0,0>
#declare T_SubRnd1 = .1
#declare T_SubRnd2 = 0.4
#include "Tree_V03.inc"



// Tree 3
#declare T_Seed1 = 5
#declare T_Seed2 = 6
#declare T_Seed3 = 7
#declare T_Seed4 = 8
#declare T_Subdivision = 2
#declare T_Trunk = false
#declare T_Branch = 6
#declare T_SBranch = 4
#declare T_Gravity = 30
#declare T_Contort = 8
#declare T_InitLenght = 2
#declare T_MinLenght = 1.8
#declare T_InitSize = .08
#declare T_MinSize = .02
#declare T_LeafObj = Leaf
#declare T_Translate = <4,0,-5>
#declare T_SubRnd1 = .4
#declare T_SubRnd2 = 0.4
#include "Tree_V03.inc"

