
// Persistence of Vision Ray Tracer Include Description File
// File: .Inc
// Vers: 3
// Desc: Try to Build a Normal Tree
// Date: Jun 1997
// Auth: Gianluca Terenziani
//
// Parameters     Type     Default     Description
// T_Start        <n,n,n>  <0,0,0>     Start Position
// T_Direction    <n,n,n>  <0,1,0>     Growing Direction
// T_Subdivision  n        3           Levels of Branch
// T_Branch       n        3           Branch per level
// T_SBranch      n        1           Subdivision of single Branch
// T_InitSize     n.n      0.2         Initial Size
// T_MinSize      n.n      0.02        Min Size (Final Size)
// T_InitLenght   n.n      3           Initial Lenght
// T_MinLenght    n.n      1.5         Min Lenght (Final Lenght)
// T_LeafObj      "??"     NONE        Name of Leaf Object (Se the Add Leaf Option)
// T_Seed1         n.n      1           Randomizer Seed
// T_Seed2         n.n      2           Randomizer Seed
// T_Seed3         n.n      3           Randomizer Seed
// T_Trunk        bool     true        Flag, True for Generating Trunk
// T_Contort      n.n      10          Massimo valore di contorsione suddivisione rami (n/T_Contort)
// T_Texture      texture  vedi .txt   Texture del tronco
// T_Translate    <n,n,n>  NONE
// T_Rotate       <n,n,n>  NONE
// T_Size         <n,n,n>  NONE
// T_SubRnd1      n.n      .2          SubBranch of level + 1
// T_SubRnd2      n.n      .05         SubBranch of level + 2
// ***************************************************
// Checking if this is first loop

#ifndef (_FirstLoop) #declare _FirstLoop = false #end

#if(_FirstLoop = false)

   #warning "Tree Start, Checking Trunk...\n"

   #declare _FirstLoop = true

// ***************************************************
// Checking Parameters

   #ifndef (T_Start)       #declare T_Start         = <0,0,0>   #end
   #ifndef (T_Direction)   #declare T_Direction     = <0,1,0>   #end
   #ifndef (T_Subdivision)
      #declare T_Subdivision   = 3
   #else #if (T_Subdivision > 6)
      #warning "Subdivision too high, adjusted to max (6)"
      #declare T_Subdivision = 6
   #end #end
   #ifndef (T_Branch)      #declare T_Branch        = 3         #end
   #ifndef (T_SBranch)     #declare T_SBranch       = 1         #end
   #ifndef (T_InitSize)    #declare T_InitSize      = 0.2       #end
   #ifndef (T_MinSize)     #declare T_MinSize       = 0.02      #end
   #ifndef (T_InitLenght)  #declare T_InitLenght    = 3         #end
   #ifndef (T_MinLenght)   #declare T_MinLenght     = 1.5       #end
   #ifndef (T_LeafObj)
      #declare _UseLeaf = false
   #else
      #declare _UseLeaf = true
   #end
   #ifndef (T_Seed1)        #declare T_Seed1          = 1         #end
   #ifndef (T_Seed2)        #declare T_Seed2          = 2         #end
   #ifndef (T_Seed3)        #declare T_Seed3          = 3         #end
   #ifndef (T_Seed4)        #declare T_Seed4          = 4         #end
   #ifndef (T_Trunk)        #declare T_Trunk          = true      #end
   #ifndef (T_Contort)      #declare T_Contort        = 10        #end

   #ifndef (T_Texture)
      #declare T_Texture = texture { pigment { color rgb <1,.7,.5> } finish {crand 0.3 } }
   #end

   #ifndef (T_SubRnd1)
      #declare T_SubRnd1 = .2
   #end
   #ifndef (T_SubRnd2)
      #declare T_SubRnd2 = .05
   #end

// ***************************************************
// Internal constant
   #ifndef (T_Gravity)
      #declare T_Gravity = 20
   #else #if (T_Gravity > 100)
      #warning "Gravity too high, adjusted to Default 20"
      #declare T_Gravity = 20
   #end #end


   #declare _SubBranch = true       // flag, if true can generate subbranch
// ***************************************************
// Internal variables
   // Randomizer 1
   #declare _R1 = seed(T_Seed1)
   #declare _R2 = seed(T_Seed2)
   #declare _R3 = seed(T_Seed3)
   #declare _R4 = seed(T_Seed4)

   // Current Level, 0 = Trunk
   #declare _CurrentLevel = 0
   // Init Size of Sphere
   #declare _InitSize = T_InitSize
   // End size of sphere
   #declare _EndSize  = ((T_InitSize - T_MinSize) / (T_Subdivision + 1)) * (T_Subdivision - _CurrentLevel - 1) + T_MinSize
      #if (_EndSize < 0.000001)
          #declare _EndSize = T_MinSize
      #end
   // Lenght of vector
   #declare _Lenght = ((T_InitLenght - T_MinLenght) / (T_Subdivision + 1)) * (T_Subdivision - _CurrentLevel) + T_MinLenght
   // Start coordinates
   #declare _Start = T_Start
   // Direction (+randomly)
   #declare _Direction = T_Direction
   // Rotation
   // Comment for Trunk
//   #declare _Direction = <_Direction.x + rand(_R1)-.5, _Direction.y - ((rand(_R1) * (T_Gravity / 20)) + (T_Gravity / 100)),_Direction.z + rand(_R1)-.5>
   #declare _Direction = vnormalize(_Direction)
   // End coordinates
   #declare _End = _Start + _Direction * _Lenght

// ***************************************************
// generating
   union
   {
   // Check if Trunk is needed
   #if (T_Trunk)
//       #declare Err_Vect = _Direction
//       #include "dbgvect.inc"

//       #declare Err_Flt = _CurrentLevel
//       #include "dbgflt.inc"

      #if (T_SBranch > 1)
         sphere { _Start, _InitSize }
         #declare _SBCount = 0
         #declare _SBStart = _Start
         #declare _SBStartR = _InitSize
         #declare _SBMaxRndx = (_End.x - _Start.x) / T_Contort
         #declare _SBMaxRndy = (_End.y - _Start.y) / T_Contort
         #declare _SBMaxRndz = (_End.z - _Start.z) / T_Contort
         #while (_SBCount < T_SBranch - 1)
            #declare _SBCount = _SBCount + 1
            #declare _SBEnd = <_Start.x + ((_End.x - _Start.x) / T_SBranch) * _SBCount + ((rand(_R2) * _SBMaxRndx) - (_SBMaxRndx / 2)), _Start.y + ((_End.y - _Start.y) / T_SBranch) * _SBCount + ((rand(_R2) * _SBMaxRndy) - (_SBMaxRndy / 2)), _Start.z + ((_End.z - _Start.z) / T_SBranch) * _SBCount + ((rand(_R2) * _SBMaxRndz) - (_SBMaxRndz / 2))>
            #declare _SBEndR = ((_InitSize - _EndSize) / T_SBranch) * (T_SBranch - _SBCount) + _EndSize
            cone {_SBStart, _SBStartR, _SBEnd, _SBEndR}
            sphere { _SBEnd, _SBEndR }
            #if (_SubBranch)
               #if (rand(_R4) < T_SubRnd1)
                  #declare _SubBranch = false
                  // #warning "SB FALSE"

                  #declare _SSBCount = _SBCount
                  #declare _SSBStart = _SBStart
                  #declare _SSBStartR = _SBStartR
                  #declare _SSBMaxRndx = _SBMaxRndx
                  #declare _SSBMaxRndy = _SBMaxRndy
                  #declare _SSBMaxRndz = _SBMaxRndz
                  #declare _SSBEnd = _SBEnd
                  #declare _SSBEndR = _SBEndR
                  #declare _SStart = _Start
                  #declare _SDirection = _Direction

                  #declare _SInitLenght = T_InitLenght
                  #declare _SMinLenght = T_MinLenght

                  #declare T_InitLenght = (T_InitLenght / T_SBranch) * (T_SBranch - _SBCount)
                  #declare T_MinLenght = T_InitLenght / 2

                  #declare _SInitSize = _InitSize
                  #declare _SEndSize = _EndSize
                  #declare _SLenght = _Lenght
                  #declare _SEnd = _End

                  #declare _Start = _SBEnd
                  #declare _Direction = <-_Direction.x,_Direction.y,-_Direction.z>

                  #include "Tree_V03.inc"

                  #declare T_InitLenght = _SInitLenght
                  #declare T_MinLenght = _SMinLenght

                  #declare _Start = _SStart
                  #declare _InitSize = _SInitSize
                  #declare _EndSize = _SEndSize
                  #declare _Lenght = _SLenght
                  #declare _End = _SEnd

                  #declare _Direction = _SDirection

                  // Restore _FirstLoop
                  #declare _FirstLoop = true

                  // Restore current Level
                  #declare _CurrentLevel = _CurrentLevel - 1

                  #declare _SBCount = _SSBCount
                  #declare _SBStart = _SSBStart
                  #declare _SBStartR = _SSBStartR
                  #declare _SBMaxRndx = _SSBMaxRndx
                  #declare _SBMaxRndy = _SSBMaxRndy
                  #declare _SBMaxRndz = _SSBMaxRndz
                  #declare _SBEnd = _SSBEnd
                  #declare _SBEndR = _SSBEndR

                  #declare _SubBranch = true
                  // #warning " SB TRUE\n"

               #else #if (rand(_R4) < T_SubRnd2)
                  #declare _SubBranch = false

                  #declare _SSBCount = _SBCount
                  #declare _SSBStart = _SBStart
                  #declare _SSBStartR = _SBStartR
                  #declare _SSBMaxRndx = _SBMaxRndx
                  #declare _SSBMaxRndy = _SBMaxRndy
                  #declare _SSBMaxRndz = _SBMaxRndz
                  #declare _SSBEnd = _SBEnd
                  #declare _SSBEndR = _SBEndR
                  #declare _SStart = _Start
                  #declare _SDirection = _Direction

                  #declare _SInitLenght = T_InitLenght
                  #declare _SMinLenght = T_MinLenght

                  #declare T_InitLenght = (T_InitLenght / T_SBranch) * (T_SBranch - _SBCount)
                  #declare T_MinLenght = T_InitLenght / 2

                  #declare _Start = _SBEnd
                  #declare _SInitSize = _InitSize
                  #declare _SEndSize = _EndSize
                  #declare _SLenght = _Lenght
                  #declare _SEnd = _End
                  #declare _Direction = <-_Direction.x,_Direction.y,-_Direction.z>

                  #declare _CurrentLevel = _CurrentLevel + 1

                  #include "Tree_V03.inc"

                  #declare T_InitLenght = _SInitLenght
                  #declare T_MinLenght = _SMinLenght

                  #declare _Start = _SStart
                  #declare _InitSize = _SInitSize
                  #declare _EndSize = _SEndSize
                  #declare _Lenght = _SLenght
                  #declare _End = _SEnd

                  #declare _Direction = _SDirection

                  // Restore _FirstLoop
                  #declare _FirstLoop = true

                  // Restore current Level
                  #declare _CurrentLevel = _CurrentLevel - 2

                  #declare _SBCount = _SSBCount
                  #declare _SBStart = _SSBStart
                  #declare _SBStartR = _SSBStartR
                  #declare _SBMaxRndx = _SSBMaxRndx
                  #declare _SBMaxRndy = _SSBMaxRndy
                  #declare _SBMaxRndz = _SSBMaxRndz
                  #declare _SBEnd = _SSBEnd
                  #declare _SBEndR = _SSBEndR

                  #declare _SubBranch = true
               #end #end
            #end


            #declare _SBStart = _SBEnd
            #declare _SBStartR = _SBEndR
         #end
         cone {_SBStart, _SBStartR, _End, _EndSize}
         sphere { _End, _EndSize }
      #else
         sphere { _Start, _InitSize }
         cone {_Start, _InitSize, _End, _EndSize}
         sphere { _End, _EndSize }
      #end
      // Reposition _Start, Recalculate _InitSize, _EndSize
      #declare _Start = _End
   #end

   // Saving information
   #declare _Save0Start = _Start
   #declare _Save0Direction = _Direction

   // Loop through Branch
   #declare _BCount = 1
   #while (_BCount <= T_Branch)
      // Save Counter
      #switch (_CurrentLevel)
      #case (0)
         #declare _Save0Count = _BCount
         #break
      #case (1)
         #declare _Save1Count = _BCount
         #break
      #case (2)
         #declare _Save2Count = _BCount
         #break
      #case (3)
         #declare _Save3Count = _BCount
         #break
      #case (4)
         #declare _Save4Count = _BCount
         #break
      #case (5)
         #declare _Save5Count = _BCount
         #break
      #case (6)
         #declare _Save6Count = _BCount
         #break
      #end

      // RECURSE !!!!!
      #include "Tree_V03.inc"

      // Restore Start and Direction
      #declare _Start = _Save0Start
      #declare _Direction = _Save0Direction

      // Restore _FirstLoop
      #declare _FirstLoop = true

      // Restore current Level
      #declare _CurrentLevel = _CurrentLevel - 1

      // Restore and increase Counter
      #switch (_CurrentLevel)
      #case (0)
         #declare _BCount = _Save0Count + 1
         #break
      #case (1)
         #declare _BCount = _Save1Count + 1
         #break
      #case (2)
         #declare _BCount = _Save2Count + 1
         #break
      #case (3)
         #declare _BCount = _Save3Count + 1
         #break
      #case (4)
         #declare _BCount = _Save4Count + 1
         #break
      #case (5)
         #declare _BCount = _Save5Count + 1
         #break
      #case (6)
         #declare _BCount = _Save6Count + 1
         #break
      #end
   #end
   texture {T_Texture}

   #ifdef (T_Rotate)
          rotate T_Rotate
   #end
   #ifdef (T_Size)
          scale T_Size
   #end
   #ifdef (T_Translate)
          translate T_Translate
   #end
   }
#else
// ***************************************************
// Not in first loop, only recursion
   // Increase current Level
   #declare _CurrentLevel = _CurrentLevel + 1
   #if (_CurrentLevel <= T_Subdivision)
      // Message
      // #warning concat("Creating Branch of Level ", str(_CurrentLevel,1,0)," of ", str(T_Subdivision,1,0), "\n")
   
      // Init size of sphere
      #declare _InitSize  = ((T_InitSize - T_MinSize) / (T_Subdivision + 1)) * (T_Subdivision - _CurrentLevel ) + T_MinSize
      // End size of sphere
      #declare _EndSize  = ((T_InitSize - T_MinSize) / (T_Subdivision + 1)) * (T_Subdivision - _CurrentLevel - 1) + T_MinSize
      #if (_EndSize < 0.000001)
          #declare _EndSize = T_MinSize
      #end
      // Lenght of vector
      #declare _Lenght = ((T_InitLenght - T_MinLenght) / (T_Subdivision + 1)) * (T_Subdivision - _CurrentLevel) + T_MinLenght
      // Rotation
      // old : <_Direction.x + (rand(_R1)-.5) * 2, _Direction.y - ((rand(_R1) * (T_Gravity / 20)) + (T_Gravity / 100)),_Direction.z + (rand(_R1)-.5)*2>
      #declare _Direction = <_Direction.x + (rand(_R1)-.5) * 2, _Direction.y - ((rand(_R1) * (T_Gravity / 50)) - (9.8 / T_Gravity)),_Direction.z + (rand(_R1)-.5)*2>
      #declare _Direction = vnormalize(_Direction)
      // End coordinates
      #declare _End = _Start + _Direction * _Lenght

// ***************************************************
// generating
      #if (T_SBranch > 1)
//         sphere { _Start, _InitSize }
         #declare _SBCount = 0
         #declare _SBStart = _Start
         #declare _SBStartR = _InitSize
         #declare _SBMaxRndx = (_End.x - _Start.x) / T_Contort
         #declare _SBMaxRndy = (_End.y - _Start.y) / T_Contort
         #declare _SBMaxRndz = (_End.z - _Start.z) / T_Contort
         #while (_SBCount < T_SBranch - 1)
            #declare _SBCount = _SBCount + 1
            #declare _SBEnd = <_Start.x + ((_End.x - _Start.x) / T_SBranch) * _SBCount + ((rand(_R2) * _SBMaxRndx) - (_SBMaxRndx / 2)), _Start.y + ((_End.y - _Start.y) / T_SBranch) * _SBCount + ((rand(_R2) * _SBMaxRndy) - (_SBMaxRndy / 2)), _Start.z + ((_End.z - _Start.z) / T_SBranch) * _SBCount + ((rand(_R2) * _SBMaxRndz) - (_SBMaxRndz / 2))>
            #declare _SBEndR = ((_InitSize - _EndSize) / T_SBranch) * (T_SBranch - _SBCount) + _EndSize
            cone {_SBStart, _SBStartR, _SBEnd, _SBEndR}
            sphere { _SBEnd, _SBEndR }

            #if (_SubBranch)
               #if (rand(_R4) < T_SubRnd1)
                  #declare _SubBranch = false
                  // #warning "SB FALSE"

                  #declare _SSBCount = _SBCount
                  #declare _SSBStart = _SBStart
                  #declare _SSBStartR = _SBStartR
                  #declare _SSBMaxRndx = _SBMaxRndx
                  #declare _SSBMaxRndy = _SBMaxRndy
                  #declare _SSBMaxRndz = _SBMaxRndz
                  #declare _SSBEnd = _SBEnd
                  #declare _SSBEndR = _SBEndR
                  #declare _SStart = _Start
                  #declare _SDirection = _Direction

                  #declare _SInitLenght = T_InitLenght
                  #declare _SMinLenght = T_MinLenght

                  #declare T_InitLenght = (T_InitLenght / T_SBranch) * (T_SBranch - _SBCount)
                  #declare T_MinLenght = T_InitLenght / 2

                  #declare _SInitSize = _InitSize
                  #declare _SEndSize = _EndSize
                  #declare _SLenght = _Lenght
                  #declare _SEnd = _End

                  #declare _Start = _SBEnd
                  #declare _Direction = <-_Direction.x,_Direction.y,-_Direction.z>

                  #include "Tree_V03.inc"

                  #declare T_InitLenght = _SInitLenght
                  #declare T_MinLenght = _SMinLenght

                  #declare _Start = _SStart
                  #declare _InitSize = _SInitSize
                  #declare _EndSize = _SEndSize
                  #declare _Lenght = _SLenght
                  #declare _End = _SEnd

                  #declare _Direction = _SDirection

                  // Restore _FirstLoop
                  #declare _FirstLoop = true

                  // Restore current Level
                  #declare _CurrentLevel = _CurrentLevel - 1

                  #declare _SBCount = _SSBCount
                  #declare _SBStart = _SSBStart
                  #declare _SBStartR = _SSBStartR
                  #declare _SBMaxRndx = _SSBMaxRndx
                  #declare _SBMaxRndy = _SSBMaxRndy
                  #declare _SBMaxRndz = _SSBMaxRndz
                  #declare _SBEnd = _SSBEnd
                  #declare _SBEndR = _SSBEndR

                  #declare _SubBranch = true
                  // #warning " SB TRUE\n"

               #else #if (rand(_R4) < T_SubRnd2)
                  #declare _SubBranch = false

                  #declare _SSBCount = _SBCount
                  #declare _SSBStart = _SBStart
                  #declare _SSBStartR = _SBStartR
                  #declare _SSBMaxRndx = _SBMaxRndx
                  #declare _SSBMaxRndy = _SBMaxRndy
                  #declare _SSBMaxRndz = _SBMaxRndz
                  #declare _SSBEnd = _SBEnd
                  #declare _SSBEndR = _SBEndR
                  #declare _SStart = _Start
                  #declare _SDirection = _Direction

                  #declare _SInitLenght = T_InitLenght
                  #declare _SMinLenght = T_MinLenght

                  #declare T_InitLenght = (T_InitLenght / T_SBranch) * (T_SBranch - _SBCount)
                  #declare T_MinLenght = T_InitLenght / 2

                  #declare _Start = _SBEnd
                  #declare _SInitSize = _InitSize
                  #declare _SEndSize = _EndSize
                  #declare _SLenght = _Lenght
                  #declare _SEnd = _End
                  #declare _Direction = <-_Direction.x,_Direction.y,-_Direction.z>

                  #declare _CurrentLevel = _CurrentLevel + 1

                  #include "Tree_V03.inc"

                  #declare T_InitLenght = _SInitLenght
                  #declare T_MinLenght = _SMinLenght

                  #declare _Start = _SStart
                  #declare _InitSize = _SInitSize
                  #declare _EndSize = _SEndSize
                  #declare _Lenght = _SLenght
                  #declare _End = _SEnd

                  #declare _Direction = _SDirection

                  // Restore _FirstLoop
                  #declare _FirstLoop = true

                  // Restore current Level
                  #declare _CurrentLevel = _CurrentLevel - 2

                  #declare _SBCount = _SSBCount
                  #declare _SBStart = _SSBStart
                  #declare _SBStartR = _SSBStartR
                  #declare _SBMaxRndx = _SSBMaxRndx
                  #declare _SBMaxRndy = _SSBMaxRndy
                  #declare _SBMaxRndz = _SSBMaxRndz
                  #declare _SBEnd = _SSBEnd
                  #declare _SBEndR = _SSBEndR

                  #declare _SubBranch = true
               #end #end
            #end

               // Leaf if is needed
            #if ((_CurrentLevel >= T_Subdivision-1) & _UseLeaf)
               // Use Leaf
               object {
                  T_LeafObj
                  scale (rand(_R3) * .2 + .9)
//                  rotate <0, 0, rand(_R3) * 60 + 60>
                  rotate <rand(_R3) * 60 + 60,0,rand(_R3) * 120 - 60>
                  rotate <0, rand(_R3) * 360, 0>
   //            rotate <rand(_R3) * 60 + 60, rand(_R3) * 360, rand(_R3) * 60 + 60>
                  translate _SBEnd
               }
            #end
            #declare _SBStart = _SBEnd
            #declare _SBStartR = _SBEndR
         #end
         cone {_SBStart, _SBStartR, _End, _EndSize}
         sphere { _End, _EndSize }
         #if ((_CurrentLevel = T_Subdivision) & _UseLeaf)
            // Use Leaf
            object {
            T_LeafObj
            scale (rand(_R3) * .2 + .9)
//                  rotate <0, 0, rand(_R3) * 60 + 60>
                  rotate <rand(_R3) * 60 + 60,0,rand(_R3) * 120 - 60>
                  rotate <0, rand(_R3) * 360, 0>
   //            rotate <rand(_R3) * 60 + 60, rand(_R3) * 360, rand(_R3) * 60 + 60>
            translate _End
            }
         #end

      #else
//         sphere { _Start, _InitSize }
         cone {_Start, _InitSize, _End, _EndSize}
         sphere { _End, _EndSize }
         #if ((_CurrentLevel = T_Subdivision) & _UseLeaf)
            // Use Leaf
            object {
            T_LeafObj
            scale (rand(_R3) * .2 + .9)
//                  rotate <0, 0, rand(_R3) * 60 + 60>
                  rotate <rand(_R3) * 60 + 60,0,rand(_R3) * 120 - 60>
                  rotate <0, rand(_R3) * 360, 0>
   //            rotate <rand(_R3) * 60 + 60, rand(_R3) * 360, rand(_R3) * 60 + 60>
            translate _End
            }
         #end
      #end
      // Reposition _Start, Recalculate _InitSize, _EndSize
      #declare _Start = _End


      // Loop through Branch
      #declare _BCount = 1
      #while (_BCount <= T_Branch)
         // Save Counter
      #switch (_CurrentLevel)
      #case (0)
         #declare _Save0Count = _BCount
         #declare _Save0Start = _Start
         #declare _Save0Direction = _Direction
         #break
      #case (1)
         #declare _Save1Count = _BCount
         #declare _Save1Start = _Start
         #declare _Save1Direction = _Direction
         #break
      #case (2)
         #declare _Save2Count = _BCount
         #declare _Save2Start = _Start
         #declare _Save2Direction = _Direction
         #break
      #case (3)
         #declare _Save3Count = _BCount
         #declare _Save3Start = _Start
         #declare _Save3Direction = _Direction
         #break
      #case (4)
         #declare _Save4Count = _BCount
         #declare _Save4Start = _Start
         #declare _Save4Direction = _Direction
         #break
      #case (5)
         #declare _Save5Count = _BCount
         #declare _Save5Start = _Start
         #declare _Save5Direction = _Direction
         #break
      #case (6)
         #declare _Save6Count = _BCount
         #declare _Save6Start = _Start
         #declare _Save6Direction = _Direction
         #break
      #end

      // RECURSE !!!!!
      #include "Tree_V03.inc"

      // Restore _FirstLoop
      #declare _FirstLoop = true

      // Restore current Level
      #declare _CurrentLevel = _CurrentLevel - 1

      // Restore and increase Counter
      #switch (_CurrentLevel)
      #case (0)
         #declare _BCount = _Save0Count + 1
         #declare _Start = _Save0Start
         #declare _Direction = _Save0Direction
         #break
      #case (1)
         #declare _BCount = _Save1Count + 1
         #declare _Start = _Save1Start
         #declare _Direction = _Save1Direction
         #break
      #case (2)
         #declare _BCount = _Save2Count + 1
         #declare _Start = _Save2Start
         #declare _Direction = _Save2Direction
         #break
      #case (3)
         #declare _BCount = _Save3Count + 1
         #declare _Start = _Save3Start
         #declare _Direction = _Save3Direction
         #break
      #case (4)
         #declare _BCount = _Save4Count + 1
         #declare _Start = _Save4Start
         #declare _Direction = _Save4Direction
         #break
      #case (5)
         #declare _BCount = _Save5Count + 1
         #declare _Start = _Save5Start
         #declare _Direction = _Save5Direction
         #break
      #case (6)
         #declare _BCount = _Save6Count + 1
         #declare _Start = _Save6Start
         #declare _Direction = _Save6Direction
         #break
      #end
      #end
   #end
#end

#declare _FirstLoop = false


